import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_form/nuif_form.dart';
import 'package:nui_form/nuif_form_component.dart';
import 'package:nui_core/code/nui_code_util.dart';

class NUIFColumnItem extends NUIFormContainer{
  NUIFColumnItem({
    @required List<NUIFormComponent> items,
    @required String inpId,
    this.divider:0.0,
    this.padding,
    double width,
    double height,
    Alignment align,
    EdgeInsets margin,
    double weight}): super(items: items, inpId : inpId, inpType: ComponentType.COLUMN, width: width, height: height, weight: weight, margin: margin, align: align);
  double divider;
  EdgeInsets padding;

  @override
  Widget toWidget({AllButtonClickListener allButtonClickListener, AllTextChangeListener allTextChangeListener, AllItemSelectListener allItemSelectListener, Key key}) {
    return NUIFColumnLayout(item: this, key: key, allTextChangeListener: allTextChangeListener, allButtonClickListener: allButtonClickListener, allItemSelectListener: allItemSelectListener,);
  }
}

class NUIFColumnLayout extends StatefulWidget {
  NUIFColumnLayout({@required this.item, this.allTextChangeListener, this.allButtonClickListener, this.allItemSelectListener, Key key}): super(key: key);
  final NUIFColumnItem item;
  final AllTextChangeListener allTextChangeListener;
  final AllButtonClickListener allButtonClickListener;
  final AllItemSelectListener allItemSelectListener;

  @override
  _NUIFColumnLayoutState createState() => _NUIFColumnLayoutState();
}

class _NUIFColumnLayoutState extends NUIFState<NUIFColumnLayout, NUIFColumnItem> {

  @override
  void initState(){
    this.item = widget.item;
    super.initState();
  }

  Widget _renderComponent(NUIFormComponent comp){
    return Container(
      child: comp.toWidget(allTextChangeListener: widget.allTextChangeListener, allButtonClickListener: widget.allButtonClickListener, allItemSelectListener: widget.allItemSelectListener, key: widget.key),
      width: comp.width,
      height: comp.height,
    );
  }

  List<Widget> _renderAllComponents(List<NUIFormComponent> items, double divider){
    List<Widget> views = List();
    bool addDivider = divider != null && divider > 0;
    NUICodeUtil.loop(items, (data, position, isFirst, isLast) {
      if(position != 0 && addDivider){
        views.add(Divider(height: item.divider ?? 0, thickness: 0, color: Colors.transparent,));
      }
      views.add(_renderComponent(data));
    });
    return views;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: widget.item.padding,
      child: Column(
        key: widget.key,
        mainAxisSize: MainAxisSize.min,
        children: _renderAllComponents(item.items, item.divider),
      ),
    );
  }
}
