import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_form/nuif_form.dart';
import 'package:nui_form/nuif_form_component.dart';

typedef OnItemSelectListener = void Function(NUIFormSelection value); //Clicked on next/done

class NUIFRadioStyle{
  NUIFRadioStyle({
    this.textStyle,
    this.primaryIcon,
    this.primaryIconChecked,
    this.padding,
    this.textAlign,
    this.divider: 5,
  });

  TextStyle textStyle;
  TextAlign textAlign;
  EdgeInsets padding;
  double divider;
  Widget primaryIcon;
  Widget primaryIconChecked;

  Widget getPrimaryIcon(){
    if(primaryIcon != null) return primaryIcon;
    return Icon(Icons.radio_button_off);
  }

  Widget getPrimaryIconChecked(){
    if(primaryIconChecked != null) return primaryIconChecked;
    return Icon(Icons.radio_button_checked);
  }
}

class NUIFRadioItem extends NUIFormComponent{
  NUIFRadioItem({
    @required String inpId,
    double width,
    double height,
    double weight,
    bool enabled: true,
    @required this.selections,
    this.enabledStyle,
    this.disabledStyle,
    this.selectedItem,
    this.itemSelectListener,
    this.isHorizontal: false,
    EdgeInsets margin,
    Alignment align,
  }) : super(inpId: inpId, inpType:ComponentType.RADIO, width: width, height : height, weight: weight, enabled: enabled, margin: margin, align: align);

  NUIFSelection selectedItem;
  List<NUIFSelection> selections;
  OnItemSelectListener itemSelectListener;
  NUIFRadioStyle enabledStyle;
  NUIFRadioStyle disabledStyle;
  bool isHorizontal = false;

  void onItemSelectedListener(AllItemSelectListener listener, NUIFSelection selection){
    selectedItem = selection;
    NUIFormValue value = NUIFormValue(inpId: selection.getId());
    value.value = NUIFormSelection(value: selectedItem?.getCaption(), code: selectedItem?.getValue());
    this.formValue = value;
    listener(inpId, value.value, true);
  }

  bool isSelected(NUIFSelection selection){
    return match(selection?.getId(), selectedItem?.getId());
  }

  @override
  Widget toWidget({AllButtonClickListener allButtonClickListener, AllTextChangeListener allTextChangeListener, AllItemSelectListener allItemSelectListener, Key key}) {
    return NUIFRadio(item: this, allItemSelectListener: allItemSelectListener, key: key);
  }

  @override
  void extractValue(Map<String, NUIFormValue> map){
    map[inpId] = this.selectedItem != null ? NUIFormValue(inpId: inpId, value: NUIFormSelection(code: inpId, value: selectedItem.getValue())) : null;
  }

  @override
  bool validate(bool validate){
    return true;
  }

  NUIFormValue getValue(){
    return this.selectedItem != null ? NUIFormValue(inpId: inpId, value: NUIFormSelection(code: inpId, value: selectedItem.getValue())) : null;
  }
}

class NUIFRadio extends StatefulWidget {
  NUIFRadio({@required this.item, this.allItemSelectListener, key}): super(key: key);
  final NUIFRadioItem item;
  final AllItemSelectListener allItemSelectListener;
  @override
  _NUIFRadioState createState() => _NUIFRadioState();
}

class _NUIFRadioState extends NUIFState<NUIFRadio, NUIFRadioItem> {

  @override
  void initState(){
    this.item = widget.item;
    super.initState();
  }

  void onRadioSelected(NUIFSelection selection){
    setState(() {
      item.onItemSelectedListener(widget.allItemSelectListener, selection);
    });
  }

  Widget _generateRadio(NUIFSelection selection){
    bool isEnabled = selection.selectable();
    NUIFRadioStyle style = isEnabled ? item.enabledStyle : item.disabledStyle ?? item.enabledStyle;

    final title = Text(
        selection.getCaption(),
        style: style?.textStyle,
        textAlign: style?.textAlign
    );

    final checkButton = IconButton(
      icon: item.isSelected(selection) ? style?.getPrimaryIconChecked() ?? Icon(Icons.radio_button_checked) : style?.getPrimaryIcon() ?? Icon(Icons.radio_button_off),
      enableFeedback: item.enabled,
      onPressed: (){
        if(!item.enabled) return;
        onRadioSelected(selection);
      },
    );

//    final radio = Padding(
//      padding: style.padding,
//      child: Radio(
//        activeColor: style.,
//        toggleable: item.enabled,
//        value: selection.getId(),
//        groupValue: item.selectedItem?.getId(),
//        onChanged: (String value){
//          onRadioSelected(selection);
//        }
//      ),
//    );

    return NUIRow(
      divider: VerticalDivider(width: style.divider, color: NUIColors.NUITransparent),
      children: [
        checkButton,
        title
      ],
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisSize: MainAxisSize.min,
    );
  }

  List<Widget> generateRadioItems(){
    List<Widget> radios = item.selections.map((e) {
      return _generateRadio(e);
    }).toList();
    return radios;
  }

  Widget _bindWithItem(NUIFRadioItem item){
    bool isEnabled = item.enabled;

    if(item.isHorizontal){
      return Row(
        mainAxisAlignment: MainAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: generateRadioItems(),
      );
    }
    else{
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: generateRadioItems(),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return item.wrapWidget(_bindWithItem(item));
  }
}
